@@ -32,12 +32,14 @@ import hashlib
 import errno
 import tempfile
 import functools
-import operator
 import re
 import contextlib
 import io
 import importlib
 import itertools
+import datetime
+from numbers import Number
+from typing import Union
 
 with contextlib.suppress(ImportError):
     import win32security
@@ -49,6 +51,8 @@ with contextlib.suppress(ImportError):
     import grp
 
 from . import matchers
+from . import masks
+from . import classes
 from .py37compat import best_realpath, lru_cache
 
 
@@ -57,44 +61,23 @@ __all__ = ['Path', 'TempDir']
 
 LINESEPS = ['\r\n', '\r', '\n']
 U_LINESEPS = LINESEPS + ['\u0085', '\u2028', '\u2029']
-NEWLINE = re.compile('|'.join(LINESEPS))
+B_NEWLINE = re.compile('|'.join(LINESEPS).encode())
 U_NEWLINE = re.compile('|'.join(U_LINESEPS))
-NL_END = re.compile(r'(?:{0})$'.format(NEWLINE.pattern))
-U_NL_END = re.compile(r'(?:{0})$'.format(U_NEWLINE.pattern))
+B_NL_END = re.compile(B_NEWLINE.pattern + b'$')
+U_NL_END = re.compile(U_NEWLINE.pattern + '$')
+
+_default_linesep = object()
+
+
+def _make_timestamp_ns(value: Union[Number, datetime.datetime]) -> Number:
+    timestamp_s = value if isinstance(value, Number) else value.timestamp()
+    return int(timestamp_s * 10**9)
 
 
 class TreeWalkWarning(Warning):
     pass
 
 
-# from jaraco.functools
-def compose(*funcs):
-    compose_two = lambda f1, f2: lambda *args, **kwargs: f1(f2(*args, **kwargs))  # noqa
-    return functools.reduce(compose_two, funcs)
-
-
-class ClassProperty(property):
-    def __get__(self, cls, owner):
-        return self.fget.__get__(None, owner)()
-
-
-class multimethod:
-    """
-    Acts like a classmethod when invoked from the class and like an
-    instancemethod when invoked from the instance.
-    """
-
-    def __init__(self, func):
-        self.func = func
-
-    def __get__(self, instance, owner):
-        return (
-            functools.partial(self.func, owner)
-            if instance is None
-            else functools.partial(self.func, owner, instance)
-        )
-
-
 class Traversal:
     """
     Wrap a walk result to customize the traversal.
@@ -131,6 +114,14 @@ class Traversal:
             traverse = functools.partial(self.follow, item)
 
 
+def _strip_newlines(lines):
+    r"""
+    >>> list(_strip_newlines(['Hello World\r\n', 'foo']))
+    ['Hello World', 'foo']
+    """
+    return (U_NL_END.sub('', line) for line in lines)
+
+
 class Path(str):
     """
     Represents a filesystem path.
@@ -167,7 +158,7 @@ class Path(str):
         ns = {'module': module}
         return type(subclass_name, bases, ns)
 
-    @ClassProperty
+    @classes.ClassProperty
     @classmethod
     def _next_class(cls):
         """
@@ -182,14 +173,9 @@ class Path(str):
 
     # Adding a Path and a string yields a Path.
     def __add__(self, more):
-        try:
-            return self._next_class(super(Path, self).__add__(more))
-        except TypeError:  # Python bug
-            return NotImplemented
+        return self._next_class(super(Path, self).__add__(more))
 
     def __radd__(self, other):
-        if not isinstance(other, str):
-            return NotImplemented
         return self._next_class(other.__add__(self))
 
     # The / operator joins Paths.
@@ -228,9 +214,6 @@ class Path(str):
     def __exit__(self, *_):
         os.chdir(self._old_dir)
 
-    def __fspath__(self):
-        return self
-
     @classmethod
     def getcwd(cls):
         """Return the current working directory as a path object.
@@ -373,7 +356,7 @@ class Path(str):
         .. seealso:: :func:`os.path.splitdrive`
         """
         drive, rel = self.module.splitdrive(self)
-        return self._next_class(drive), rel
+        return self._next_class(drive), self._next_class(rel)
 
     def splitext(self):
         """Return two-tuple of ``.stripext()`` and ``.ext``.
@@ -398,20 +381,7 @@ class Path(str):
         """
         return self.splitext()[0]
 
-    def splitunc(self):
-        unc, rest = self.module.splitunc(self)
-        return self._next_class(unc), rest
-
-    @property
-    def uncshare(self):
-        """
-        The UNC mount point for this path.
-        This is empty for paths on local drives.
-        """
-        unc, r = self.module.splitunc(self)
-        return self._next_class(unc)
-
-    @multimethod
+    @classes.multimethod
     def joinpath(cls, first, *others):
         """
         Join first to zero or more :class:`Path` components,
@@ -421,9 +391,7 @@ class Path(str):
 
         .. seealso:: :func:`os.path.join`
         """
-        if not isinstance(first, cls):
-            first = cls(first)
-        return first._next_class(first.module.join(first, *others))
+        return cls._next_class(cls.module.join(first, *others))
 
     def splitall(self):
         r"""Return a list of the path components in this path.
@@ -441,6 +409,10 @@ class Path(str):
         return list(self._parts())
 
     def parts(self):
+        """
+        >>> Path('/foo/bar/baz').parts()
+        (Path('/'), 'foo', 'bar', 'baz')
+        """
         return tuple(self._parts())
 
     def _parts(self):
@@ -576,6 +548,7 @@ class Path(str):
                 do_traverse = traverse()
             except Exception as exc:
                 errors(f"Unable to access '{child}': {exc}")
+                continue
 
             if do_traverse:
                 for item in child.walk(errors=errors, match=match):
@@ -683,11 +656,7 @@ class Path(str):
         Default behavior is to overwrite any existing file.
         Call ``p.write_bytes(bytes, append=True)`` to append instead.
         """
-        if append:
-            mode = 'ab'
-        else:
-            mode = 'wb'
-        with self.open(mode) as f:
+        with self.open('ab' if append else 'wb') as f:
             f.write(bytes)
 
     def read_text(self, encoding=None, errors=None):
@@ -727,24 +696,22 @@ class Path(str):
 
         Parameters:
 
-          `text` - str/unicode - The text to be written.
+          `text` - str/bytes - The text to be written.
 
-          `encoding` - str - The Unicode encoding that will be used.
-              This is ignored if `text` isn't a Unicode string.
+          `encoding` - str - The text encoding used.
 
           `errors` - str - How to handle Unicode encoding errors.
               Default is ``'strict'``.  See ``help(unicode.encode)`` for the
-              options.  This is ignored if `text` isn't a Unicode
-              string.
+              options.  Ignored if `text` isn't a Unicode string.
 
           `linesep` - keyword argument - str/unicode - The sequence of
               characters to be used to mark end-of-line.  The default is
-              :data:`os.linesep`.  You can also specify ``None`` to
-              leave all newlines as they are in `text`.
+              :data:`os.linesep`.  Specify ``None`` to
+              use newlines unmodified.
 
           `append` - keyword argument - bool - Specifies what to do if
               the file already exists (``True``: append to the end of it;
-              ``False``: overwrite it.)  The default is ``False``.
+              ``False``: overwrite it).  The default is ``False``.
 
 
         --- Newline handling.
@@ -754,18 +721,13 @@ class Path(str):
         end-of-line sequence (see :data:`os.linesep`; on Windows, for example,
         the end-of-line marker is ``'\r\n'``).
 
-        If you don't like your platform's default, you can override it
-        using the `linesep=` keyword argument.  If you specifically want
-        ``write_text()`` to preserve the newlines as-is, use ``linesep=None``.
-
-        This applies to Unicode text the same as to 8-bit text, except
-        there are three additional standard Unicode end-of-line sequences:
-        ``u'\x85'``, ``u'\r\x85'``, and ``u'\u2028'``.
-
-        (This is slightly different from when you open a file for
-        writing with ``fopen(filename, "w")`` in C or ``open(filename, 'w')``
-        in Python.)
+        To override the platform's default, pass the `linesep=`
+        keyword argument. To preserve the newlines as-is, pass
+        ``linesep=None``.
 
+        This handling applies to Unicode text and bytes, except
+        with Unicode, additional non-ASCII newlines are recognized:
+        ``\x85``, ``\r\x85``, and ``\u2028``.
 
         --- Unicode
 
@@ -777,31 +739,37 @@ class Path(str):
         specified `encoding` (or the default encoding if `encoding`
         isn't specified).  The `errors` argument applies only to this
         conversion.
-
         """
         if isinstance(text, str):
             if linesep is not None:
                 text = U_NEWLINE.sub(linesep, text)
-            text = text.encode(encoding or sys.getdefaultencoding(), errors)
+            bytes = text.encode(encoding or sys.getdefaultencoding(), errors)
         else:
+            warnings.warn(
+                "Writing bytes in write_text is deprecated",
+                DeprecationWarning,
+                stacklevel=1,
+            )
             assert encoding is None
-            text = NEWLINE.sub(linesep, text)
-        self.write_bytes(text, append=append)
+            if linesep is not None:
+                text = B_NEWLINE.sub(linesep.encode(), text)
+            bytes = text
+        self.write_bytes(bytes, append=append)
 
-    def lines(self, encoding=None, errors='strict', retain=True):
+    def lines(self, encoding=None, errors=None, retain=True):
         r"""Open this file, read all lines, return them in a list.
 
         Optional arguments:
             `encoding` - The Unicode encoding (or character set) of
-                the file.  The default is ``None``, meaning the content
-                of the file is read as 8-bit characters and returned
-                as a list of (non-Unicode) str objects.
-            `errors` - How to handle Unicode errors; see help(str.decode)
-                for the options.  Default is ``'strict'``.
-            `retain` - If ``True``, retain newline characters; but all newline
-                character combinations (``'\r'``, ``'\n'``, ``'\r\n'``) are
-                translated to ``'\n'``.  If ``False``, newline characters are
-                stripped off.  Default is ``True``.
+                the file.  The default is ``None``, meaning use
+                ``locale.getpreferredencoding()``.
+            `errors` - How to handle Unicode errors; see
+                `open <https://docs.python.org/3/library/functions.html#open>`_
+                for the options.  Default is ``None`` meaning "strict".
+            `retain` - If ``True`` (default), retain newline characters,
+                but translate all newline
+                characters to ``\n``.  If ``False``, newline characters are
+                omitted.
 
         .. seealso:: :meth:`text`
         """
@@ -809,7 +777,12 @@ class Path(str):
         return text.splitlines(retain)
 
     def write_lines(
-        self, lines, encoding=None, errors='strict', linesep=os.linesep, append=False
+        self,
+        lines,
+        encoding=None,
+        errors='strict',
+        linesep=_default_linesep,
+        append=False,
     ):
         r"""Write the given lines of text to this file.
 
@@ -826,7 +799,7 @@ class Path(str):
             `errors` - How to handle errors in Unicode encoding.  This
                 also applies only to Unicode strings.
 
-            linesep - The desired line-ending.  This line-ending is
+            linesep - (deprecated) The desired line-ending.  This line-ending is
                 applied to every line.  If a line already has any
                 standard line ending (``'\r'``, ``'\n'``, ``'\r\n'``,
                 ``u'\x85'``, ``u'\r\x85'``, ``u'\u2028'``), that will
@@ -838,24 +811,21 @@ class Path(str):
 
         Use the keyword argument ``append=True`` to append lines to the
         file.  The default is to overwrite the file.
-
-        .. warning ::
-
-            When you use this with Unicode data, if the encoding of the
-            existing data in the file is different from the encoding
-            you specify with the `encoding=` parameter, the result is
-            mixed-encoding data, which can really confuse someone trying
-            to read the file later.
         """
-        with self.open('ab' if append else 'wb') as f:
-            for line in lines:
-                isUnicode = isinstance(line, str)
-                if linesep is not None:
-                    pattern = U_NL_END if isUnicode else NL_END
-                    line = pattern.sub('', line) + linesep
-                if isUnicode:
-                    line = line.encode(encoding or sys.getdefaultencoding(), errors)
-                f.write(line)
+        mode = 'a' if append else 'w'
+        with self.open(mode, encoding=encoding, errors=errors, newline='') as f:
+            f.writelines(self._replace_linesep(lines, linesep))
+
+    @staticmethod
+    def _replace_linesep(lines, linesep):
+        if linesep != _default_linesep:
+            warnings.warn("linesep is deprecated", DeprecationWarning, stacklevel=3)
+        else:
+            linesep = os.linesep
+        if linesep is None:
+            return lines
+
+        return (line + linesep for line in _strip_newlines(lines))
 
     def read_md5(self):
         """Calculate the md5 hash for this file.
@@ -903,7 +873,12 @@ class Path(str):
     # bound. Playing it safe and wrapping them all in method calls.
 
     def isabs(self):
-        """.. seealso:: :func:`os.path.isabs`"""
+        """
+        >>> Path('.').isabs()
+        False
+
+        .. seealso:: :func:`os.path.isabs`
+        """
         return self.module.isabs(self)
 
     def exists(self):
@@ -923,25 +898,43 @@ class Path(str):
         return self.module.islink(self)
 
     def ismount(self):
-        """.. seealso:: :func:`os.path.ismount`"""
+        """
+        >>> Path('.').ismount()
+        False
+
+        .. seealso:: :func:`os.path.ismount`
+        """
         return self.module.ismount(self)
 
     def samefile(self, other):
         """.. seealso:: :func:`os.path.samefile`"""
-        if not hasattr(self.module, 'samefile'):
-            other = Path(other).realpath().normpath().normcase()
-            return self.realpath().normpath().normcase() == other
         return self.module.samefile(self, other)
 
     def getatime(self):
         """.. seealso:: :attr:`atime`, :func:`os.path.getatime`"""
         return self.module.getatime(self)
 
+    def set_atime(self, value):
+        mtime_ns = self.stat().st_atime_ns
+        self.utime(ns=(_make_timestamp_ns(value), mtime_ns))
+
     atime = property(
         getatime,
+        set_atime,
         None,
-        None,
-        """ Last access time of the file.
+        """
+        Last access time of the file.
+
+        >>> Path('.').atime > 0
+        True
+
+        Allows setting:
+
+        >>> some_file = Path(getfixture('tmp_path')).joinpath('file.txt').touch()
+        >>> MST = datetime.timezone(datetime.timedelta(hours=-7))
+        >>> some_file.atime = datetime.datetime(1976, 5, 7, 10, tzinfo=MST)
+        >>> some_file.atime
+        200336400.0
 
         .. seealso:: :meth:`getatime`, :func:`os.path.getatime`
         """,
@@ -951,11 +944,24 @@ class Path(str):
         """.. seealso:: :attr:`mtime`, :func:`os.path.getmtime`"""
         return self.module.getmtime(self)
 
+    def set_mtime(self, value):
+        atime_ns = self.stat().st_atime_ns
+        self.utime(ns=(atime_ns, _make_timestamp_ns(value)))
+
     mtime = property(
         getmtime,
+        set_mtime,
         None,
-        None,
-        """ Last-modified time of the file.
+        """
+        Last modified time of the file.
+
+        Allows setting:
+
+        >>> some_file = Path(getfixture('tmp_path')).joinpath('file.txt').touch()
+        >>> MST = datetime.timezone(datetime.timedelta(hours=-7))
+        >>> some_file.mtime = datetime.datetime(1976, 5, 7, 10, tzinfo=MST)
+        >>> some_file.mtime
+        200336400.0
 
         .. seealso:: :meth:`getmtime`, :func:`os.path.getmtime`
         """,
@@ -989,38 +995,45 @@ class Path(str):
         """,
     )
 
-    if hasattr(os, 'access'):
+    def access(self, *args, **kwargs):
+        """
+        Return does the real user have access to this path.
 
-        def access(self, mode):
-            """Return ``True`` if current user has access to this path.
+        >>> Path('.').access(os.F_OK)
+        True
 
-            mode - One of the constants :data:`os.F_OK`, :data:`os.R_OK`,
-            :data:`os.W_OK`, :data:`os.X_OK`
-
-            .. seealso:: :func:`os.access`
-            """
-            return os.access(self, mode)
+        .. seealso:: :func:`os.access`
+        """
+        return os.access(self, *args, **kwargs)
 
     def stat(self):
-        """Perform a ``stat()`` system call on this path.
+        """
+        Perform a ``stat()`` system call on this path.
+
+        >>> Path('.').stat()
+        os.stat_result(...)
 
         .. seealso:: :meth:`lstat`, :func:`os.stat`
         """
         return os.stat(self)
 
     def lstat(self):
-        """Like :meth:`stat`, but do not follow symbolic links.
+        """
+        Like :meth:`stat`, but do not follow symbolic links.
+
+        >>> Path('.').lstat() == Path('.').stat()
+        True
 
         .. seealso:: :meth:`stat`, :func:`os.lstat`
         """
         return os.lstat(self)
 
-    def __get_owner_windows(self):
-        """
+    def __get_owner_windows(self):  # pragma: nocover
+        r"""
         Return the name of the owner of this file or directory. Follow
         symbolic links.
 
-        Return a name of the form ``r'DOMAIN\\User Name'``; may be a group.
+        Return a name of the form ``DOMAIN\User Name``; may be a group.
 
         .. seealso:: :attr:`owner`
         """
@@ -1031,7 +1044,7 @@ class Path(str):
         account, domain, typecode = win32security.LookupAccountSid(None, sid)
         return domain + '\\' + account
 
-    def __get_owner_unix(self):
+    def __get_owner_unix(self):  # pragma: nocover
         """
         Return the name of the owner of this file or directory. Follow
         symbolic links.
@@ -1041,15 +1054,16 @@ class Path(str):
         st = self.stat()
         return pwd.getpwuid(st.st_uid).pw_name
 
-    def __get_owner_not_implemented(self):
+    def __get_owner_not_implemented(self):  # pragma: nocover
         raise NotImplementedError("Ownership not available on this platform.")
 
-    if 'win32security' in globals():
-        get_owner = __get_owner_windows
-    elif 'pwd' in globals():
-        get_owner = __get_owner_unix
-    else:
-        get_owner = __get_owner_not_implemented
+    get_owner = (
+        __get_owner_windows
+        if 'win32security' in globals()
+        else __get_owner_unix
+        if 'pwd' in globals()
+        else __get_owner_not_implemented
+    )
 
     owner = property(
         get_owner,
@@ -1078,12 +1092,12 @@ class Path(str):
     #
     # --- Modifying operations on files and directories
 
-    def utime(self, times):
+    def utime(self, *args, **kwargs):
         """Set the access and modified times of this file.
 
         .. seealso:: :func:`os.utime`
         """
-        os.utime(self, times)
+        os.utime(self, *args, **kwargs)
         return self
 
     def chmod(self, mode):
@@ -1094,27 +1108,28 @@ class Path(str):
         .. seealso:: :func:`os.chmod`
         """
         if isinstance(mode, str):
-            mask = _multi_permission_mask(mode)
+            mask = masks.compound(mode)
             mode = mask(self.stat().st_mode)
         os.chmod(self, mode)
         return self
 
-    def chown(self, uid=-1, gid=-1):
-        """
-        Change the owner and group by names rather than the uid or gid numbers.
+    if hasattr(os, 'chown'):
 
-        .. seealso:: :func:`os.chown`
-        """
-        if hasattr(os, 'chown'):
-            if 'pwd' in globals() and isinstance(uid, str):
-                uid = pwd.getpwnam(uid).pw_uid
-            if 'grp' in globals() and isinstance(gid, str):
-                gid = grp.getgrnam(gid).gr_gid
-            os.chown(self, uid, gid)
-        else:
-            msg = "Ownership not available on this platform."
-            raise NotImplementedError(msg)
-        return self
+        def chown(self, uid=-1, gid=-1):
+            """
+            Change the owner and group by names or numbers.
+
+            .. seealso:: :func:`os.chown`
+            """
+
+            def resolve_uid(uid):
+                return uid if isinstance(uid, int) else pwd.getpwnam(uid).pw_uid
+
+            def resolve_gid(gid):
+                return gid if isinstance(gid, int) else grp.getgrnam(gid).gr_gid
+
+            os.chown(self, resolve_uid(uid), resolve_gid(gid))
+            return self
 
     def rename(self, new):
         """.. seealso:: :func:`os.rename`"""
@@ -1203,67 +1218,50 @@ class Path(str):
             self.unlink()
         return self
 
-    def unlink(self):
-        """.. seealso:: :func:`os.unlink`"""
-        os.unlink(self)
-        return self
-
-    def unlink_p(self):
-        """Like :meth:`unlink`, but does not raise an exception if the
-        file does not exist."""
-        self.remove_p()
-        return self
+    unlink = remove
+    unlink_p = remove_p
 
     # --- Links
 
-    if hasattr(os, 'link'):
+    def link(self, newpath):
+        """Create a hard link at `newpath`, pointing to this file.
 
-        def link(self, newpath):
-            """Create a hard link at `newpath`, pointing to this file.
+        .. seealso:: :func:`os.link`
+        """
+        os.link(self, newpath)
+        return self._next_class(newpath)
 
-            .. seealso:: :func:`os.link`
-            """
-            os.link(self, newpath)
-            return self._next_class(newpath)
+    def symlink(self, newlink=None):
+        """Create a symbolic link at `newlink`, pointing here.
 
-    if hasattr(os, 'symlink'):
+        If newlink is not supplied, the symbolic link will assume
+        the name self.basename(), creating the link in the cwd.
 
-        def symlink(self, newlink=None):
-            """Create a symbolic link at `newlink`, pointing here.
+        .. seealso:: :func:`os.symlink`
+        """
+        if newlink is None:
+            newlink = self.basename()
+        os.symlink(self, newlink)
+        return self._next_class(newlink)
 
-            If newlink is not supplied, the symbolic link will assume
-            the name self.basename(), creating the link in the cwd.
+    def readlink(self):
+        """Return the path to which this symbolic link points.
 
-            .. seealso:: :func:`os.symlink`
-            """
-            if newlink is None:
-                newlink = self.basename()
-            os.symlink(self, newlink)
-            return self._next_class(newlink)
+        The result may be an absolute or a relative path.
 
-    if hasattr(os, 'readlink'):
+        .. seealso:: :meth:`readlinkabs`, :func:`os.readlink`
+        """
+        return self._next_class(os.readlink(self))
 
-        def readlink(self):
-            """Return the path to which this symbolic link points.
+    def readlinkabs(self):
+        """Return the path to which this symbolic link points.
 
-            The result may be an absolute or a relative path.
+        The result is always an absolute path.
 
-            .. seealso:: :meth:`readlinkabs`, :func:`os.readlink`
-            """
-            return self._next_class(os.readlink(self))
-
-        def readlinkabs(self):
-            """Return the path to which this symbolic link points.
-
-            The result is always an absolute path.
-
-            .. seealso:: :meth:`readlink`, :func:`os.readlink`
-            """
-            p = self.readlink()
-            if p.isabs():
-                return p
-            else:
-                return (self.parent / p).abspath()
+        .. seealso:: :meth:`readlink`, :func:`os.readlink`
+        """
+        p = self.readlink()
+        return p if p.isabs() else (self.parent / p).abspath()
 
     # High-level functions from shutil
     # These functions will be bound to the instance such that
@@ -1344,15 +1342,15 @@ class Path(str):
 
     if hasattr(os, 'chroot'):
 
-        def chroot(self):
+        def chroot(self):  # pragma: nocover
             """.. seealso:: :func:`os.chroot`"""
             os.chroot(self)
 
     if hasattr(os, 'startfile'):
 
-        def startfile(self):
+        def startfile(self, *args, **kwargs):  # pragma: nocover
             """.. seealso:: :func:`os.startfile`"""
-            os.startfile(self)
+            os.startfile(self, *args, **kwargs)
             return self
 
     # in-place re-writing, courtesy of Martijn Pieters
@@ -1392,8 +1390,6 @@ class Path(str):
 
         Thereafter, the file at `filename` will have line numbers in it.
         """
-        import io
-
         if set(mode).intersection('wa+'):
             raise ValueError('Only read-only file modes can be used')
 
@@ -1422,8 +1418,7 @@ class Path(str):
             )
         else:
             os_mode = os.O_CREAT | os.O_WRONLY | os.O_TRUNC
-            if hasattr(os, 'O_BINARY'):
-                os_mode |= os.O_BINARY
+            os_mode |= getattr(os, 'O_BINARY', 0)
             fd = os.open(self, os_mode, perm)
             writable = io.open(
                 fd,
@@ -1450,7 +1445,7 @@ class Path(str):
         finally:
             backup_fn.remove_p()
 
-    @ClassProperty
+    @classes.ClassProperty
     @classmethod
     def special(cls):
         """
@@ -1582,7 +1577,7 @@ class Multi:
     def __iter__(self):
         return iter(map(self._next_class, self.split(os.pathsep)))
 
-    @ClassProperty
+    @classes.ClassProperty
     @classmethod
     def _next_class(cls):
         """
@@ -1611,7 +1606,7 @@ class TempDir(Path):
     .. seealso:: :func:`tempfile.mkdtemp`
     """
 
-    @ClassProperty
+    @classes.ClassProperty
     @classmethod
     def _next_class(cls):
         return Path
@@ -1634,82 +1629,6 @@ class TempDir(Path):
         self.rmtree()
 
 
-def _multi_permission_mask(mode):
-    """
-    Support multiple, comma-separated Unix chmod symbolic modes.
-
-    >>> _multi_permission_mask('a=r,u+w')(0) == 0o644
-    True
-    """
-
-    def compose(f, g):
-        return lambda *args, **kwargs: g(f(*args, **kwargs))
-
-    return functools.reduce(compose, map(_permission_mask, mode.split(',')))
-
-
-def _permission_mask(mode):
-    """
-    Convert a Unix chmod symbolic mode like ``'ugo+rwx'`` to a function
-    suitable for applying to a mask to affect that change.
-
-    >>> mask = _permission_mask('ugo+rwx')
-    >>> mask(0o554) == 0o777
-    True
-
-    >>> _permission_mask('go-x')(0o777) == 0o766
-    True
-
-    >>> _permission_mask('o-x')(0o445) == 0o444
-    True
-
-    >>> _permission_mask('a+x')(0) == 0o111
-    True
-
-    >>> _permission_mask('a=rw')(0o057) == 0o666
-    True
-
-    >>> _permission_mask('u=x')(0o666) == 0o166
-    True
-
-    >>> _permission_mask('g=')(0o157) == 0o107
-    True
-    """
-    # parse the symbolic mode
-    parsed = re.match('(?P<who>[ugoa]+)(?P<op>[-+=])(?P<what>[rwx]*)$', mode)
-    if not parsed:
-        raise ValueError("Unrecognized symbolic mode", mode)
-
-    # generate a mask representing the specified permission
-    spec_map = dict(r=4, w=2, x=1)
-    specs = (spec_map[perm] for perm in parsed.group('what'))
-    spec = functools.reduce(operator.or_, specs, 0)
-
-    # now apply spec to each subject in who
-    shift_map = dict(u=6, g=3, o=0)
-    who = parsed.group('who').replace('a', 'ugo')
-    masks = (spec << shift_map[subj] for subj in who)
-    mask = functools.reduce(operator.or_, masks)
-
-    op = parsed.group('op')
-
-    # if op is -, invert the mask
-    if op == '-':
-        mask ^= 0o777
-
-    # if op is =, retain extant values for unreferenced subjects
-    if op == '=':
-        masks = (0o7 << shift_map[subj] for subj in who)
-        retain = functools.reduce(operator.or_, masks) ^ 0o777
-
-    op_map = {
-        '+': operator.or_,
-        '-': operator.and_,
-        '=': lambda mask, target: target & retain ^ mask,
-    }
-    return functools.partial(op_map[op], mask)
-
-
 class Handlers:
     def strict(msg):
         raise
